@@ -66,7 +66,7 @@ impl super::Queue {
         gl.disable(glow::BLEND);
         gl.disable(glow::CULL_FACE);
         gl.disable(glow::POLYGON_OFFSET_FILL);
-        if self.features.contains(wgt::Features::DEPTH_CLAMPING) {
+        if self.features.contains(wgt::Features::DEPTH_CLIP_CONTROL) {
             gl.disable(glow::DEPTH_CLAMP);
         }
     }
@@ -202,46 +202,40 @@ impl super::Queue {
                 gl.dispatch_compute_indirect(indirect_offset as i32);
             }
             C::ClearBuffer {
-                dst,
+                ref dst,
                 dst_target,
                 ref range,
-            } => {
-                gl.bind_buffer(glow::COPY_READ_BUFFER, Some(self.zero_buffer));
-                gl.bind_buffer(dst_target, Some(dst));
-                let mut dst_offset = range.start;
-                while dst_offset < range.end {
-                    let size = (range.end - dst_offset).min(super::ZERO_BUFFER_SIZE as u64);
-                    gl.copy_buffer_sub_data(
-                        glow::COPY_READ_BUFFER,
-                        dst_target,
-                        0,
-                        dst_offset as i32,
-                        size as i32,
-                    );
-                    dst_offset += size;
+            } => match dst.raw {
+                Some(buffer) => {
+                    gl.bind_buffer(glow::COPY_READ_BUFFER, Some(self.zero_buffer));
+                    gl.bind_buffer(dst_target, Some(buffer));
+                    let mut dst_offset = range.start;
+                    while dst_offset < range.end {
+                        let size = (range.end - dst_offset).min(super::ZERO_BUFFER_SIZE as u64);
+                        gl.copy_buffer_sub_data(
+                            glow::COPY_READ_BUFFER,
+                            dst_target,
+                            0,
+                            dst_offset as i32,
+                            size as i32,
+                        );
+                        dst_offset += size;
+                    }
                 }
-            }
-            C::ClearTexture {
-                dst: _,
-                dst_target: _,
-                subresource_range: _,
-            } => {
-                // Should EXT_clear_texture when possible.
-                // https://www.khronos.org/registry/OpenGL/extensions/EXT/EXT_clear_texture.txt
-                // But support is not very widespread. Need to fallback to do zero_buffer copies
-
-                // TODO: Need to invoke calls into CopyBufferToTexture using zero_buffer.
-                // To do that determine how many rows zero_buffer can fill and then chunk the texture up
-                // (do *not* repeat the exact logic of CopyBufferToTexture, it's way too much!)
-                //unimplemented!("texture clearing for GLES is not implemented yet");
-            }
+                None => {
+                    dst.data.as_ref().unwrap().lock().unwrap().as_mut_slice()
+                        [range.start as usize..range.end as usize]
+                        .fill(0);
+                }
+            },
             C::CopyBufferToBuffer {
-                src,
+                ref src,
                 src_target,
-                dst,
+                ref dst,
                 dst_target,
                 copy,
             } => {
+                let copy_src_target = glow::COPY_READ_BUFFER;
                 let is_index_buffer_only_element_dst = !self
                     .shared
                     .private_caps
@@ -249,44 +243,49 @@ impl super::Queue {
                     && dst_target == glow::ELEMENT_ARRAY_BUFFER
                     || src_target == glow::ELEMENT_ARRAY_BUFFER;
 
-                let copy_src_target = glow::COPY_READ_BUFFER;
-
                 // WebGL not allowed to copy data from other targets to element buffer and can't copy element data to other buffers
                 let copy_dst_target = if is_index_buffer_only_element_dst {
                     glow::ELEMENT_ARRAY_BUFFER
                 } else {
                     glow::COPY_WRITE_BUFFER
                 };
+                let size = copy.size.get() as usize;
+                match (src.raw, dst.raw) {
+                    (Some(ref src), Some(ref dst)) => {
+                        gl.bind_buffer(copy_src_target, Some(*src));
+                        gl.bind_buffer(copy_dst_target, Some(*dst));
+                        gl.copy_buffer_sub_data(
+                            copy_src_target,
+                            copy_dst_target,
+                            copy.src_offset as _,
+                            copy.dst_offset as _,
+                            copy.size.get() as _,
+                        );
+                    }
+                    (Some(src), None) => {
+                        let mut data = dst.data.as_ref().unwrap().lock().unwrap();
+                        let dst_data = &mut data.as_mut_slice()
+                            [copy.dst_offset as usize..copy.dst_offset as usize + size];
 
-                gl.bind_buffer(copy_src_target, Some(src));
-                gl.bind_buffer(copy_dst_target, Some(dst));
-
-                //TODO: remove this slow path completely
-                // https://github.com/gfx-rs/wgpu/issues/2031
-                if is_index_buffer_only_element_dst {
-                    let mut buffer_data = vec![0; copy.size.get() as usize];
-                    gl.get_buffer_sub_data(
-                        copy_src_target,
-                        copy.src_offset as i32,
-                        &mut buffer_data,
-                    );
-                    gl.buffer_sub_data_u8_slice(
-                        copy_dst_target,
-                        copy.dst_offset as i32,
-                        &buffer_data,
-                    );
-                } else {
-                    gl.copy_buffer_sub_data(
-                        copy_src_target,
-                        copy_dst_target,
-                        copy.src_offset as _,
-                        copy.dst_offset as _,
-                        copy.size.get() as _,
-                    );
+                        gl.bind_buffer(copy_src_target, Some(src));
+                        gl.get_buffer_sub_data(copy_src_target, copy.src_offset as i32, dst_data);
+                    }
+                    (None, Some(dst)) => {
+                        let data = src.data.as_ref().unwrap().lock().unwrap();
+                        let src_data = &data.as_slice()
+                            [copy.src_offset as usize..copy.src_offset as usize + size];
+                        gl.bind_buffer(copy_dst_target, Some(dst));
+                        gl.buffer_sub_data_u8_slice(
+                            copy_dst_target,
+                            copy.dst_offset as i32,
+                            src_data,
+                        );
+                    }
+                    (None, None) => {
+                        todo!()
+                    }
                 }
-
                 gl.bind_buffer(copy_src_target, None);
-
                 if is_index_buffer_only_element_dst {
                     gl.bind_buffer(glow::ELEMENT_ARRAY_BUFFER, self.current_index_buffer);
                 } else {
@@ -349,7 +348,7 @@ impl super::Queue {
                 }
             }
             C::CopyBufferToTexture {
-                src,
+                ref src,
                 src_target: _,
                 dst,
                 dst_target,
@@ -358,7 +357,6 @@ impl super::Queue {
             } => {
                 let format_info = dst_format.describe();
                 let format_desc = self.shared.describe_texture_format(dst_format);
-
                 let row_texels = copy.buffer_layout.bytes_per_row.map_or(0, |bpr| {
                     format_info.block_dimensions.0 as u32 * bpr.get()
                         / format_info.block_size as u32
@@ -367,14 +365,26 @@ impl super::Queue {
                     .buffer_layout
                     .rows_per_image
                     .map_or(0, |rpi| format_info.block_dimensions.1 as u32 * rpi.get());
+
+                gl.bind_texture(dst_target, Some(dst));
                 gl.pixel_store_i32(glow::UNPACK_ROW_LENGTH, row_texels as i32);
                 gl.pixel_store_i32(glow::UNPACK_IMAGE_HEIGHT, column_texels as i32);
-                gl.bind_buffer(glow::PIXEL_UNPACK_BUFFER, Some(src));
-                gl.bind_texture(dst_target, Some(dst));
-
+                let mut unbind_unpack_buffer = false;
                 if format_info.block_dimensions == (1, 1) {
-                    let unpack_data =
-                        glow::PixelUnpackData::BufferOffset(copy.buffer_layout.offset as u32);
+                    let buffer_data;
+                    let unpack_data = match src.raw {
+                        Some(buffer) => {
+                            gl.bind_buffer(glow::PIXEL_UNPACK_BUFFER, Some(buffer));
+                            unbind_unpack_buffer = true;
+                            glow::PixelUnpackData::BufferOffset(copy.buffer_layout.offset as u32)
+                        }
+                        None => {
+                            buffer_data = src.data.as_ref().unwrap().lock().unwrap();
+                            let src_data =
+                                &buffer_data.as_slice()[copy.buffer_layout.offset as usize..];
+                            glow::PixelUnpackData::Slice(src_data)
+                        }
+                    };
                     match dst_target {
                         glow::TEXTURE_3D | glow::TEXTURE_2D_ARRAY => {
                             gl.tex_sub_image_3d(
@@ -405,7 +415,6 @@ impl super::Queue {
                             );
                         }
                         glow::TEXTURE_CUBE_MAP => {
-                            let offset = copy.buffer_layout.offset as u32;
                             gl.tex_sub_image_2d(
                                 CUBEMAP_FACES[copy.texture_base.array_layer as usize],
                                 copy.texture_base.mip_level as i32,
@@ -415,7 +424,7 @@ impl super::Queue {
                                 copy.size.height as i32,
                                 format_desc.external,
                                 format_desc.data_type,
-                                glow::PixelUnpackData::BufferOffset(offset),
+                                unpack_data,
                             );
                         }
                         glow::TEXTURE_CUBE_MAP_ARRAY => {
@@ -441,9 +450,23 @@ impl super::Queue {
                         copy.buffer_layout.rows_per_image.map_or(1, |rpi| rpi.get())
                             * copy.buffer_layout.bytes_per_row.map_or(1, |bpr| bpr.get());
                     let offset = copy.buffer_layout.offset as u32;
-                    let unpack_data = glow::CompressedPixelUnpackData::BufferRange(
-                        offset..offset + bytes_per_image,
-                    );
+
+                    let buffer_data;
+                    let unpack_data = match src.raw {
+                        Some(buffer) => {
+                            gl.bind_buffer(glow::PIXEL_UNPACK_BUFFER, Some(buffer));
+                            unbind_unpack_buffer = true;
+                            glow::CompressedPixelUnpackData::BufferRange(
+                                offset..offset + bytes_per_image,
+                            )
+                        }
+                        None => {
+                            buffer_data = src.data.as_ref().unwrap().lock().unwrap();
+                            let src_data = &buffer_data.as_slice()
+                                [(offset as usize)..(offset + bytes_per_image) as usize];
+                            glow::CompressedPixelUnpackData::Slice(src_data)
+                        }
+                    };
                     match dst_target {
                         glow::TEXTURE_3D | glow::TEXTURE_2D_ARRAY => {
                             gl.compressed_tex_sub_image_3d(
@@ -480,9 +503,7 @@ impl super::Queue {
                                 copy.size.width as i32,
                                 copy.size.height as i32,
                                 format_desc.internal,
-                                glow::CompressedPixelUnpackData::BufferRange(
-                                    offset..offset + bytes_per_image,
-                                ),
+                                unpack_data,
                             );
                         }
                         glow::TEXTURE_CUBE_MAP_ARRAY => {
@@ -503,13 +524,15 @@ impl super::Queue {
                         _ => unreachable!(),
                     }
                 }
-                gl.bind_buffer(glow::PIXEL_UNPACK_BUFFER, None);
+                if unbind_unpack_buffer {
+                    gl.bind_buffer(glow::PIXEL_UNPACK_BUFFER, None);
+                }
             }
             C::CopyTextureToBuffer {
                 src,
                 src_target,
                 src_format,
-                dst,
+                ref dst,
                 dst_target: _,
                 ref copy,
             } => {
@@ -531,8 +554,6 @@ impl super::Queue {
                     .map_or(copy.size.width, |bpr| {
                         bpr.get() / format_info.block_size as u32
                     });
-                gl.pixel_store_i32(glow::PACK_ROW_LENGTH, row_texels as i32);
-                gl.bind_buffer(glow::PIXEL_PACK_BUFFER, Some(dst));
 
                 gl.bind_framebuffer(glow::READ_FRAMEBUFFER, Some(self.copy_fbo));
                 //TODO: handle cubemap copies
@@ -554,6 +575,20 @@ impl super::Queue {
                         copy.texture_base.mip_level as i32,
                     );
                 }
+                let mut buffer_data;
+                let unpack_data = match dst.raw {
+                    Some(buffer) => {
+                        gl.pixel_store_i32(glow::PACK_ROW_LENGTH, row_texels as i32);
+                        gl.bind_buffer(glow::PIXEL_PACK_BUFFER, Some(buffer));
+                        glow::PixelPackData::BufferOffset(copy.buffer_layout.offset as u32)
+                    }
+                    None => {
+                        buffer_data = dst.data.as_ref().unwrap().lock().unwrap();
+                        let dst_data =
+                            &mut buffer_data.as_mut_slice()[copy.buffer_layout.offset as usize..];
+                        glow::PixelPackData::Slice(dst_data)
+                    }
+                };
                 gl.read_pixels(
                     copy.texture_base.origin.x as i32,
                     copy.texture_base.origin.y as i32,
@@ -561,7 +596,7 @@ impl super::Queue {
                     copy.size.height as i32,
                     format_desc.external,
                     format_desc.data_type,
-                    glow::PixelPackData::BufferOffset(copy.buffer_layout.offset as u32),
+                    unpack_data,
                 );
             }
             C::SetIndexBuffer(buffer) => {
@@ -576,7 +611,7 @@ impl super::Queue {
             }
             C::CopyQueryResults {
                 ref query_range,
-                dst,
+                ref dst,
                 dst_target,
                 dst_offset,
             } => {
@@ -589,8 +624,17 @@ impl super::Queue {
                     self.temp_query_results.as_ptr() as *const u8,
                     self.temp_query_results.len() * mem::size_of::<u64>(),
                 );
-                gl.bind_buffer(dst_target, Some(dst));
-                gl.buffer_sub_data_u8_slice(dst_target, dst_offset as i32, query_data);
+                match dst.raw {
+                    Some(buffer) => {
+                        gl.bind_buffer(dst_target, Some(buffer));
+                        gl.buffer_sub_data_u8_slice(dst_target, dst_offset as i32, query_data);
+                    }
+                    None => {
+                        let data = &mut dst.data.as_ref().unwrap().lock().unwrap();
+                        let len = query_data.len().min(data.len());
+                        data[..len].copy_from_slice(&query_data[..len]);
+                    }
+                }
             }
             C::ResetFramebuffer => {
                 gl.bind_framebuffer(glow::DRAW_FRAMEBUFFER, Some(self.draw_fbo));
@@ -881,8 +925,9 @@ impl super::Queue {
                 } else {
                     gl.disable(glow::CULL_FACE);
                 }
-                if self.features.contains(wgt::Features::DEPTH_CLAMPING) {
-                    if state.clamp_depth {
+                if self.features.contains(wgt::Features::DEPTH_CLIP_CONTROL) {
+                    //Note: this is a bit tricky, since we are controlling the clip, not the clamp.
+                    if state.unclipped_depth {
                         gl.enable(glow::DEPTH_CLAMP);
                     } else {
                         gl.disable(glow::DEPTH_CLAMP);
