@@ -79,6 +79,12 @@ impl crate::Adapter<super::Api> for super::Adapter {
                     | Tfc::COLOR_ATTACHMENT
                     | Tfc::COLOR_ATTACHMENT_BLEND
             }
+            Tf::R16Unorm | Tf::R16Snorm => {
+                Tfc::SAMPLED_LINEAR
+                    | Tfc::STORAGE
+                    | Tfc::COLOR_ATTACHMENT
+                    | Tfc::COLOR_ATTACHMENT_BLEND
+            }
             Tf::Rg8Unorm | Tf::Rg8Snorm => {
                 Tfc::SAMPLED_LINEAR
                     | Tfc::STORAGE
@@ -105,6 +111,12 @@ impl crate::Adapter<super::Api> for super::Adapter {
             Tf::Rg16Uint | Tf::Rg16Sint => {
                 read_write_tier2_if | Tfc::STORAGE | Tfc::COLOR_ATTACHMENT
             }
+            Tf::Rg16Unorm | Tf::Rg16Snorm => {
+                Tfc::SAMPLED_LINEAR
+                    | Tfc::STORAGE
+                    | Tfc::COLOR_ATTACHMENT
+                    | Tfc::COLOR_ATTACHMENT_BLEND
+            }
             Tf::Rg16Float => {
                 read_write_tier2_if
                     | Tfc::SAMPLED_LINEAR
@@ -159,6 +171,12 @@ impl crate::Adapter<super::Api> for super::Adapter {
             Tf::Rgba16Uint | Tf::Rgba16Sint => {
                 read_write_tier2_if | Tfc::STORAGE | Tfc::COLOR_ATTACHMENT
             }
+            Tf::Rgba16Unorm | Tf::Rgba16Snorm => {
+                Tfc::SAMPLED_LINEAR
+                    | Tfc::STORAGE
+                    | Tfc::COLOR_ATTACHMENT
+                    | Tfc::COLOR_ATTACHMENT_BLEND
+            }
             Tf::Rgba16Float => {
                 read_write_tier2_if
                     | Tfc::SAMPLED_LINEAR
@@ -217,14 +235,16 @@ impl crate::Adapter<super::Api> for super::Adapter {
                     Tfc::empty()
                 }
             }
-            Tf::Etc2RgbUnorm
-            | Tf::Etc2RgbUnormSrgb
-            | Tf::Etc2RgbA1Unorm
-            | Tf::Etc2RgbA1UnormSrgb
-            | Tf::EacRUnorm
-            | Tf::EacRSnorm
-            | Tf::EacRgUnorm
-            | Tf::EacRgSnorm => {
+            Tf::Etc2Rgb8Unorm
+            | Tf::Etc2Rgb8UnormSrgb
+            | Tf::Etc2Rgb8A1Unorm
+            | Tf::Etc2Rgb8A1UnormSrgb
+            | Tf::Etc2Rgba8Unorm
+            | Tf::Etc2Rgba8UnormSrgb
+            | Tf::EacR11Unorm
+            | Tf::EacR11Snorm
+            | Tf::EacRg11Unorm
+            | Tf::EacRg11Snorm => {
                 if pc.format_eac_etc {
                     Tfc::SAMPLED_LINEAR
                 } else {
@@ -785,6 +805,30 @@ impl super::PrivateCapabilities {
             } else {
                 4
             },
+            max_varying_components: if Self::supports_any(
+                device,
+                &[
+                    MTLFeatureSet::macOS_GPUFamily1_v1,
+                    MTLFeatureSet::macOS_GPUFamily2_v1,
+                ],
+            ) {
+                128
+            } else {
+                60
+            },
+            max_threads_per_group: if Self::supports_any(
+                device,
+                &[
+                    MTLFeatureSet::iOS_GPUFamily4_v2,
+                    MTLFeatureSet::iOS_GPUFamily5_v1,
+                    MTLFeatureSet::macOS_GPUFamily1_v1,
+                    MTLFeatureSet::macOS_GPUFamily2_v1,
+                ],
+            ) {
+                1024
+            } else {
+                512
+            },
             max_total_threadgroup_memory: if Self::supports_any(
                 device,
                 &[
@@ -859,7 +903,8 @@ impl super::PrivateCapabilities {
                 Self::version_at_least(major, minor, 11, 0)
             },
             //Depth clipping is supported on all macOS GPU families and iOS family 4 and later
-            supports_depth_clamping: device.supports_feature_set(MTLFeatureSet::iOS_GPUFamily4_v1)
+            supports_depth_clip_control: device
+                .supports_feature_set(MTLFeatureSet::iOS_GPUFamily4_v1)
                 || os_is_mac,
         }
     }
@@ -869,13 +914,16 @@ impl super::PrivateCapabilities {
 
         let mut features = F::empty()
             | F::TEXTURE_COMPRESSION_BC
+            | F::INDIRECT_FIRST_INSTANCE
             | F::MAPPABLE_PRIMARY_BUFFERS
             | F::VERTEX_WRITABLE_STORAGE
             | F::TEXTURE_ADAPTER_SPECIFIC_FORMAT_FEATURES
+            | F::PUSH_CONSTANTS
             | F::POLYGON_MODE_LINE
-            | F::CLEAR_COMMANDS;
+            | F::CLEAR_COMMANDS
+            | F::TEXTURE_FORMAT_16BIT_NORM;
 
-        features.set(F::DEPTH_CLAMPING, self.supports_depth_clamping);
+        features.set(F::DEPTH_CLIP_CONTROL, self.supports_depth_clip_control);
 
         features.set(
             F::TEXTURE_BINDING_ARRAY
@@ -944,6 +992,14 @@ impl super::PrivateCapabilities {
                 max_push_constant_size: 0x1000,
                 min_uniform_buffer_offset_alignment: self.buffer_alignment as u32,
                 min_storage_buffer_offset_alignment: self.buffer_alignment as u32,
+                //TODO: double-check how these match Metal feature set tables
+                max_inter_stage_shader_components: self.max_varying_components,
+                max_compute_workgroup_storage_size: self.max_total_threadgroup_memory,
+                max_compute_invocations_per_workgroup: self.max_threads_per_group,
+                max_compute_workgroup_size_x: 256,
+                max_compute_workgroup_size_y: 256,
+                max_compute_workgroup_size_z: 64,
+                max_compute_workgroups_per_dimension: 0xFFFF,
             },
             alignments: crate::Alignments {
                 buffer_copy_offset: wgt::BufferSize::new(self.buffer_alignment).unwrap(),
@@ -964,11 +1020,15 @@ impl super::PrivateCapabilities {
             Tf::R8Sint => R8Sint,
             Tf::R16Uint => R16Uint,
             Tf::R16Sint => R16Sint,
+            Tf::R16Unorm => R16Unorm,
+            Tf::R16Snorm => R16Snorm,
             Tf::R16Float => R16Float,
             Tf::Rg8Unorm => RG8Unorm,
             Tf::Rg8Snorm => RG8Snorm,
             Tf::Rg8Uint => RG8Uint,
             Tf::Rg8Sint => RG8Sint,
+            Tf::Rg16Unorm => RG16Unorm,
+            Tf::Rg16Snorm => RG16Snorm,
             Tf::R32Uint => R32Uint,
             Tf::R32Sint => R32Sint,
             Tf::R32Float => R32Float,
@@ -989,6 +1049,8 @@ impl super::PrivateCapabilities {
             Tf::Rg32Float => RG32Float,
             Tf::Rgba16Uint => RGBA16Uint,
             Tf::Rgba16Sint => RGBA16Sint,
+            Tf::Rgba16Unorm => RGBA16Unorm,
+            Tf::Rgba16Snorm => RGBA16Snorm,
             Tf::Rgba16Float => RGBA16Float,
             Tf::Rgba32Uint => RGBA32Uint,
             Tf::Rgba32Sint => RGBA32Sint,
@@ -1023,14 +1085,16 @@ impl super::PrivateCapabilities {
             Tf::Bc6hRgbUfloat => BC6H_RGBUfloat,
             Tf::Bc7RgbaUnorm => BC7_RGBAUnorm,
             Tf::Bc7RgbaUnormSrgb => BC7_RGBAUnorm_sRGB,
-            Tf::Etc2RgbUnorm => ETC2_RGB8,
-            Tf::Etc2RgbUnormSrgb => ETC2_RGB8_sRGB,
-            Tf::Etc2RgbA1Unorm => ETC2_RGB8A1,
-            Tf::Etc2RgbA1UnormSrgb => ETC2_RGB8A1_sRGB,
-            Tf::EacRUnorm => EAC_R11Unorm,
-            Tf::EacRSnorm => EAC_R11Snorm,
-            Tf::EacRgUnorm => EAC_RG11Unorm,
-            Tf::EacRgSnorm => EAC_RG11Snorm,
+            Tf::Etc2Rgb8Unorm => ETC2_RGB8,
+            Tf::Etc2Rgb8UnormSrgb => ETC2_RGB8_sRGB,
+            Tf::Etc2Rgb8A1Unorm => ETC2_RGB8A1,
+            Tf::Etc2Rgb8A1UnormSrgb => ETC2_RGB8A1_sRGB,
+            Tf::Etc2Rgba8Unorm => EAC_RGBA8,
+            Tf::Etc2Rgba8UnormSrgb => EAC_RGBA8_sRGB,
+            Tf::EacR11Unorm => EAC_R11Unorm,
+            Tf::EacR11Snorm => EAC_R11Snorm,
+            Tf::EacRg11Unorm => EAC_RG11Unorm,
+            Tf::EacRg11Snorm => EAC_RG11Snorm,
             Tf::Astc4x4RgbaUnorm => ASTC_4x4_LDR,
             Tf::Astc4x4RgbaUnormSrgb => ASTC_4x4_sRGB,
             Tf::Astc5x4RgbaUnorm => ASTC_5x4_LDR,
