@@ -1,3 +1,4 @@
+use parking_lot::Mutex;
 use std::{
     ptr,
     sync::{atomic, Arc},
@@ -174,6 +175,28 @@ impl super::Device {
                 .set_mutability(mtl::MTLMutability::Immutable);
         }
     }
+
+    pub unsafe fn texture_from_raw(
+        raw: mtl::Texture,
+        raw_format: mtl::MTLPixelFormat,
+        raw_type: mtl::MTLTextureType,
+        array_layers: u32,
+        mip_levels: u32,
+        copy_size: crate::CopyExtent,
+    ) -> super::Texture {
+        super::Texture {
+            raw,
+            raw_format,
+            raw_type,
+            array_layers,
+            mip_levels,
+            copy_size,
+        }
+    }
+
+    pub fn raw_device(&self) -> &Mutex<mtl::Device> {
+        &self.shared.device
+    }
 }
 
 impl crate::Device<super::Api> for super::Device {
@@ -204,7 +227,6 @@ impl crate::Device<super::Api> for super::Device {
         Ok(super::Buffer {
             raw,
             size: desc.size,
-            options,
         })
     }
     unsafe fn destroy_buffer(&self, _buffer: super::Buffer) {}
@@ -284,7 +306,6 @@ impl crate::Device<super::Api> for super::Device {
 
         Ok(super::Texture {
             raw,
-            format: desc.format,
             raw_format: mtl_format,
             raw_type: mtl_type,
             mip_levels: desc.mip_level_count,
@@ -308,6 +329,8 @@ impl crate::Device<super::Api> for super::Device {
             conv::map_texture_view_dimension(desc.dimension)
         };
 
+        //Note: this doesn't check properly if the mipmap level count or array layer count
+        // is explicitly set to 1.
         let raw = if raw_format == texture.raw_format
             && raw_type == texture.raw_type
             && desc.range == wgt::ImageSubresourceRange::default()
@@ -450,6 +473,7 @@ impl crate::Device<super::Api> for super::Device {
         let mut bind_group_infos = arrayvec::ArrayVec::new();
 
         // First, place the push constants
+        let mut total_push_constants = 0;
         for info in stage_data.iter_mut() {
             for pcr in desc.push_constant_ranges {
                 if pcr.stages.contains(map_naga_stage(info.stage)) {
@@ -471,6 +495,8 @@ impl crate::Device<super::Api> for super::Device {
                 info.pc_buffer = Some(info.counters.buffers);
                 info.counters.buffers += 1;
             }
+
+            total_push_constants = total_push_constants.max(info.pc_limit);
         }
 
         // Second, place the described resources
@@ -614,7 +640,13 @@ impl crate::Device<super::Api> for super::Device {
                         ..per_stage_map.cs
                     },
                 },
+                bounds_check_policies: naga::proc::BoundsCheckPolicies {
+                    index: naga::proc::BoundsCheckPolicy::ReadZeroSkipWrite,
+                    buffer: naga::proc::BoundsCheckPolicy::ReadZeroSkipWrite,
+                    image: naga::proc::BoundsCheckPolicy::ReadZeroSkipWrite,
+                },
             },
+            total_push_constants,
         })
     }
     unsafe fn destroy_pipeline_layout(&self, _pipeline_layout: super::PipelineLayout) {}
@@ -889,8 +921,8 @@ impl crate::Device<super::Api> for super::Device {
             raw_triangle_fill_mode,
             raw_front_winding: conv::map_winding(desc.primitive.front_face),
             raw_cull_mode: conv::map_cull_mode(desc.primitive.cull_mode),
-            raw_depth_clip_mode: if self.features.contains(wgt::Features::DEPTH_CLAMPING) {
-                Some(if desc.primitive.clamp_depth {
+            raw_depth_clip_mode: if self.features.contains(wgt::Features::DEPTH_CLIP_CONTROL) {
+                Some(if desc.primitive.unclipped_depth {
                     mtl::MTLDepthClipMode::Clamp
                 } else {
                     mtl::MTLDepthClipMode::Clip
