@@ -85,12 +85,12 @@ impl crate::Instance<Api> for Instance {
     ) -> Result<Surface, crate::InstanceError> {
         match has_handle.raw_window_handle() {
             #[cfg(target_os = "ios")]
-            raw_window_handle::RawWindowHandle::IOS(handle) => {
+            raw_window_handle::RawWindowHandle::UiKit(handle) => {
                 let _ = &self.managed_metal_layer_delegate;
                 Ok(Surface::from_uiview(handle.ui_view))
             }
             #[cfg(target_os = "macos")]
-            raw_window_handle::RawWindowHandle::MacOS(handle) => Ok(Surface::from_nsview(
+            raw_window_handle::RawWindowHandle::AppKit(handle) => Ok(Surface::from_nsview(
                 handle.ns_view,
                 &self.managed_metal_layer_delegate,
             )),
@@ -137,6 +137,7 @@ impl crate::Instance<Api> for Instance {
     }
 }
 
+#[allow(dead_code)]
 #[derive(Clone, Debug)]
 struct PrivateCapabilities {
     family_check: bool,
@@ -211,6 +212,8 @@ struct PrivateCapabilities {
     max_texture_layers: u64,
     max_fragment_input_components: u64,
     max_color_render_targets: u8,
+    max_varying_components: u32,
+    max_threads_per_group: u32,
     max_total_threadgroup_memory: u32,
     sample_count_mask: u8,
     supports_debug_markers: bool,
@@ -222,7 +225,7 @@ struct PrivateCapabilities {
     supports_arrays_of_textures: bool,
     supports_arrays_of_textures_write: bool,
     supports_mutability: bool,
-    supports_depth_clamping: bool,
+    supports_depth_clip_control: bool,
 }
 
 #[derive(Clone, Debug)]
@@ -230,6 +233,7 @@ struct PrivateDisabilities {
     /// Near depth is not respected properly on some Intel GPUs.
     broken_viewport_near_depth: bool,
     /// Multi-target clears don't appear to work properly on Intel GPUs.
+    #[allow(dead_code)]
     broken_layered_clear_image: bool,
 }
 
@@ -238,17 +242,11 @@ struct Settings {
     retain_command_buffer_references: bool,
 }
 
-// Using max copyable texture row
-// https://developer.apple.com/documentation/metal/mtlblitcommandencoder/1400752-copyfrombuffer?language=objc
-// "The value must be less than or equal to 32767 multiplied by the destination texture’s pixel size."
-const ZERO_BUFFER_SIZE: wgt::BufferAddress = 32767 * 16; // 512kb
-
 struct AdapterShared {
     device: Mutex<mtl::Device>,
     disabilities: PrivateDisabilities,
     private_caps: PrivateCapabilities,
     settings: Settings,
-    zero_buffer: mtl::Buffer,
 }
 
 unsafe impl Send for AdapterShared {}
@@ -259,20 +257,11 @@ impl AdapterShared {
         let private_caps = PrivateCapabilities::new(&device);
         log::debug!("{:#?}", private_caps);
 
-        // buffers created this way are zero initialized
-        // see https://developer.apple.com/documentation/metal/mtldevice/1433375-newbufferwithlength?language=objc
-        let zero_buffer = device.new_buffer(
-            ZERO_BUFFER_SIZE,
-            mtl::MTLResourceOptions::CPUCacheModeWriteCombined
-                | mtl::MTLResourceOptions::StorageModePrivate,
-        );
-
         Self {
             disabilities: PrivateDisabilities::new(&device),
             private_caps: PrivateCapabilities::new(&device),
             device: Mutex::new(device),
             settings: Settings::default(),
-            zero_buffer,
         }
     }
 }
@@ -296,7 +285,6 @@ pub struct Device {
 pub struct Surface {
     view: Option<NonNull<objc::runtime::Object>>,
     render_layer: Mutex<mtl::MetalLayer>,
-    swapchain_format: wgt::TextureFormat,
     raw_swapchain_format: mtl::MTLPixelFormat,
     extent: wgt::Extent3d,
     main_thread_id: thread::ThreadId,
@@ -407,7 +395,6 @@ impl crate::Queue<Api> for Queue {
 pub struct Buffer {
     raw: mtl::Buffer,
     size: wgt::BufferAddress,
-    options: mtl::MTLResourceOptions,
 }
 
 unsafe impl Send for Buffer {}
@@ -422,7 +409,6 @@ impl Buffer {
 #[derive(Debug)]
 pub struct Texture {
     raw: mtl::Texture,
-    format: wgt::TextureFormat,
     raw_format: mtl::MTLPixelFormat,
     raw_type: mtl::MTLTextureType,
     array_layers: u32,
@@ -538,6 +524,7 @@ pub struct PipelineLayout {
     bind_group_infos: ArrayVec<BindGroupLayoutInfo, { crate::MAX_BIND_GROUPS }>,
     push_constants_infos: MultiStageData<Option<PushConstantsInfo>>,
     total_counters: MultiStageResourceCounters,
+    total_push_constants: u32,
 }
 
 trait AsNative {
@@ -723,6 +710,7 @@ struct CommandState {
     stage_infos: MultiStageData<PipelineStageInfo>,
     storage_buffer_length_map: fxhash::FxHashMap<naga::ResourceBinding, wgt::BufferSize>,
     work_group_memory_sizes: Vec<u32>,
+    push_constants: Vec<u32>,
 }
 
 pub struct CommandEncoder {
