@@ -12,11 +12,15 @@ impl super::PrivateCapabilities {
             Tf::R8Sint => F::R8_SINT,
             Tf::R16Uint => F::R16_UINT,
             Tf::R16Sint => F::R16_SINT,
+            Tf::R16Unorm => F::R16_UNORM,
+            Tf::R16Snorm => F::R16_SNORM,
             Tf::R16Float => F::R16_SFLOAT,
             Tf::Rg8Unorm => F::R8G8_UNORM,
             Tf::Rg8Snorm => F::R8G8_SNORM,
             Tf::Rg8Uint => F::R8G8_UINT,
             Tf::Rg8Sint => F::R8G8_SINT,
+            Tf::Rg16Unorm => F::R16G16_UNORM,
+            Tf::Rg16Snorm => F::R16G16_SNORM,
             Tf::R32Uint => F::R32_UINT,
             Tf::R32Sint => F::R32_SINT,
             Tf::R32Float => F::R32_SFLOAT,
@@ -37,6 +41,8 @@ impl super::PrivateCapabilities {
             Tf::Rg32Float => F::R32G32_SFLOAT,
             Tf::Rgba16Uint => F::R16G16B16A16_UINT,
             Tf::Rgba16Sint => F::R16G16B16A16_SINT,
+            Tf::Rgba16Unorm => F::R16G16B16A16_UNORM,
+            Tf::Rgba16Snorm => F::R16G16B16A16_SNORM,
             Tf::Rgba16Float => F::R16G16B16A16_SFLOAT,
             Tf::Rgba32Uint => F::R32G32B32A32_UINT,
             Tf::Rgba32Sint => F::R32G32B32A32_SINT,
@@ -71,14 +77,16 @@ impl super::PrivateCapabilities {
             Tf::Bc6hRgbSfloat => F::BC6H_SFLOAT_BLOCK,
             Tf::Bc7RgbaUnorm => F::BC7_UNORM_BLOCK,
             Tf::Bc7RgbaUnormSrgb => F::BC7_SRGB_BLOCK,
-            Tf::Etc2RgbUnorm => F::ETC2_R8G8B8_UNORM_BLOCK,
-            Tf::Etc2RgbUnormSrgb => F::ETC2_R8G8B8_SRGB_BLOCK,
-            Tf::Etc2RgbA1Unorm => F::ETC2_R8G8B8A1_UNORM_BLOCK,
-            Tf::Etc2RgbA1UnormSrgb => F::ETC2_R8G8B8A1_SRGB_BLOCK,
-            Tf::EacRUnorm => F::EAC_R11_UNORM_BLOCK,
-            Tf::EacRSnorm => F::EAC_R11_SNORM_BLOCK,
-            Tf::EacRgUnorm => F::EAC_R11G11_UNORM_BLOCK,
-            Tf::EacRgSnorm => F::EAC_R11G11_SNORM_BLOCK,
+            Tf::Etc2Rgb8Unorm => F::ETC2_R8G8B8_UNORM_BLOCK,
+            Tf::Etc2Rgb8UnormSrgb => F::ETC2_R8G8B8_SRGB_BLOCK,
+            Tf::Etc2Rgb8A1Unorm => F::ETC2_R8G8B8A1_UNORM_BLOCK,
+            Tf::Etc2Rgb8A1UnormSrgb => F::ETC2_R8G8B8A1_SRGB_BLOCK,
+            Tf::Etc2Rgba8Unorm => F::ETC2_R8G8B8A8_UNORM_BLOCK,
+            Tf::Etc2Rgba8UnormSrgb => F::ETC2_R8G8B8A8_SRGB_BLOCK,
+            Tf::EacR11Unorm => F::EAC_R11_UNORM_BLOCK,
+            Tf::EacR11Snorm => F::EAC_R11_SNORM_BLOCK,
+            Tf::EacRg11Unorm => F::EAC_R11G11_UNORM_BLOCK,
+            Tf::EacRg11Snorm => F::EAC_R11G11_SNORM_BLOCK,
             Tf::Astc4x4RgbaUnorm => F::ASTC_4X4_UNORM_BLOCK,
             Tf::Astc4x4RgbaUnormSrgb => F::ASTC_4X4_SRGB_BLOCK,
             Tf::Astc5x4RgbaUnorm => F::ASTC_5X4_UNORM_BLOCK,
@@ -512,6 +520,20 @@ pub fn map_copy_extent(extent: &crate::CopyExtent) -> vk::Extent3D {
     }
 }
 
+pub fn map_extent_to_copy_size(
+    extent: &wgt::Extent3d,
+    dim: wgt::TextureDimension,
+) -> crate::CopyExtent {
+    crate::CopyExtent {
+        width: extent.width,
+        height: extent.height,
+        depth: match dim {
+            wgt::TextureDimension::D1 | wgt::TextureDimension::D2 => 1,
+            wgt::TextureDimension::D3 => extent.depth_or_array_layers,
+        },
+    }
+}
+
 pub fn map_subresource_range(
     range: &wgt::ImageSubresourceRange,
     texture_aspect: crate::FormatAspects,
